import logging

from rest_framework.authtoken.models import Token
from rest_framework.test import APIClient
from django.contrib.auth import get_user_model

import pytest

from greg.models import (
    Person,
    Sponsor,
    PersonIdentity,
    Role,
    OrganizationalUnit,
    PersonRole,
)

# faker spams the logs with localisation warnings
# see https://github.com/joke2k/faker/issues/753
logging.getLogger("faker").setLevel(logging.ERROR)


@pytest.fixture
def client() -> APIClient:
    user, _ = get_user_model().objects.get_or_create(username="test")
    token, _ = Token.objects.get_or_create(user=user)
    client = APIClient()
    client.credentials(HTTP_AUTHORIZATION=f"Token {token.key}")
    return client


@pytest.fixture
def person_foo() -> Person:
    return Person.objects.create(
        first_name="Foo",
        last_name="Foo",
        date_of_birth="2001-01-27",
        email="test@example.org",
        mobile_phone="123456788",
    )


@pytest.fixture
def person_bar() -> Person:
    return Person.objects.create(
        first_name="Bar",
        last_name="Bar",
        date_of_birth="2000-07-01",
        email="test2@example.org",
        mobile_phone="123456789",
    )


@pytest.fixture
def sponsor_guy() -> Sponsor:
    return Sponsor.objects.create(feide_id="guy@example.org")


@pytest.fixture
def person_foo_verified(person_foo, sponsor_guy) -> PersonIdentity:
    return PersonIdentity.objects.create(
        person=person_foo,
        type=PersonIdentity.IdentityType.PASSPORT,
        source="Test",
        value="12345",
        verified=PersonIdentity.Verified.MANUAL,
        verified_by=sponsor_guy,
        verified_when="2021-06-15",
    )


@pytest.fixture
def person_foo_not_verified(person_foo) -> PersonIdentity:
    return PersonIdentity.objects.create(
        person=person_foo,
        type=PersonIdentity.IdentityType.DRIVERS_LICENSE,
        source="Test",
        value="12345",
    )


@pytest.fixture()
def role_test_guest() -> Role:
    return Role.objects.create(type="Test Guest")


@pytest.fixture
def unit_foo() -> OrganizationalUnit:
    return OrganizationalUnit.objects.create(orgreg_id="12345", name_en="foo_unit")


@pytest.fixture
def person_foo_role(
    person_foo: Person,
    role_test_guest: Role,
    sponsor_guy: Sponsor,
    unit_foo: OrganizationalUnit,
) -> PersonRole:
    return PersonRole.objects.create(
        person=person_foo,
        role=role_test_guest,
        start_date="2021-08-02",
        end_date="2021-08-06",
        registered_by=sponsor_guy,
        unit=unit_foo,
    )
