import datetime

import pytest

from greg.models import (
    Person,
    Consent,
    PersonConsent,
)


@pytest.fixture
def person() -> Person:
    return Person.objects.create(
        first_name="Test",
        last_name="Tester",
        date_of_birth="2000-01-27",
        email="test@example.org",
        mobile_phone="123456789",
    )


@pytest.fixture()
def consent() -> Consent:
    return Consent.objects.create(
        type="it_guidelines",
        consent_name_en="IT Guidelines",
        consent_name_nb="IT Regelverk",
        consent_description_en="IT Guidelines description",
        consent_description_nb="IT Regelverk beskrivelse",
        consent_link_en="https://example.org/it_guidelines",
        user_allowed_to_change=False,
    )


@pytest.mark.django_db
def test_add_consent_to_person(person: Person, consent: Consent):
    consent_given_date = "2021-06-20"
    person.consents.add(consent, through_defaults={"consent_given_at": consent_given_date})  # type: ignore
    person_consent_links = PersonConsent.objects.filter(person_id=person.id)

    assert len(person_consent_links) == 1
    assert person_consent_links[0].person_id == person.id
    assert person_consent_links[0].consent_id == consent.id
    assert person_consent_links[0].consent_given_at == datetime.date(2021, 6, 20)


@pytest.mark.django_db
def test_add_not_acknowledged_consent_to_person(person: Person, consent: Consent):
    person.consents.add(consent)
    person_consent_links = PersonConsent.objects.filter(person_id=person.id)
    assert len(person_consent_links) == 1
    assert person_consent_links[0].person_id == person.id
    assert person_consent_links[0].consent_id == consent.id
    assert person_consent_links[0].consent_given_at is None
